@@ -82,14 +82,19 @@ def validate_sample_list(xls_fn,
                 raise Exception(f"sample and buffer not in the same row: {swell} vs {bwell}")
                 
     wdict = df[~df["Sample"].isnull()].set_index("Well").T.to_dict()
+    wlist = list(wdict.keys())
     # each well either need to have volume specified, or how to generate the sample
     mlist = {}   # source well for each new sample
     slist = {}   # total volume out of source well
-    for wn in wdict.keys():
+    for wn in wlist:
         if isinstance(wdict[wn]["Mixing"], str): 
             tl = {}
             for tt in wdict[wn]['Mixing'].strip().split(","):
                 w,v = tt.strip().split(":")
+                if w not in wlist:
+                    raise Exception(f"source well {w} is empty.")
+                if not isinstance(wdict[w]['Stock'], str):
+                    raise Exception(f"source well {w} is not designated as a stock well.")
                 if w in tl.keys():
                     raise Exception(f"{w} appears more than once in the mixing list for {wn}.")
                 tl[w] = float(v)
